# encoding: utf-8
import logging

from efa.lib.validators import (
    euhu_validator_factory,
    build_data_to_validate
)
from efa.model.participation import Participation
from efa.views.participation import (
    update_ as efa_update,
    create_ as efa_create,
    generate_controllist,
    check_and_set_fp1_personal_id)
from euhu.programs.qe import validate
from pyramid.httpexceptions import HTTPUnauthorized
from pyramid.view import view_config
from qe.model.participation import QEParticipation
from ringo.lib import security
from ringo.lib.helpers import get_action_routename
from ringo.views.helpers import (
    get_item_from_request
)

log = logging.getLogger(__name__)

_ = lambda x: x
_("The present data do not meet the requirements for an ESF evaluation of "
  "the output and result indicators. For this reason the participation will "
  "currently be exluded from all ESFevaluations. Please check and fix the "
  "data to make the participation evaluable again.")


def euhu_validator(field, data, request):
    participation = get_item_from_request(request)
    # For GET request return the cached validation state.
    if request.method == "GET" and hasattr(participation, "valid"):
        if participation.valid is not None:
            return participation.valid

    # Revalidate the participation
    data_factory = build_data_to_validate(data, participation)

    # Save the result of the validation.
    if hasattr(participation, "valid"):
        if validate(data_factory).has_violated_rules():
            participation.valid = 0
            return False
        else:
            participation.valid = 1
            return True
    return True

@view_config(route_name=get_action_routename(QEParticipation, 'create'),
             renderer='/default/create.mako',
             permission='create')
def create_(request):
    return efa_create(request, external_callbacks=[check_and_set_fp1_personal_id])

@view_config(route_name=get_action_routename(Participation, 'update'),
             renderer='/default/update.mako',
             permission='update')
def update_(request):
    validators = [euhu_validator_factory(request, euhu_validator)]
    return efa_update(request, validators)


@view_config(route_name="controllist",
             renderer="/default/list.mako")
def controllist(request):
    """Display a summary list of participations for organisation users."""
    if not security.has_role(request.user, "beneficiary"):
        raise HTTPUnauthorized
    return generate_controllist(request, QEParticipation)
